function [lk,be,Pi,P] = emcontcomp(Y,J,C,X,Bm,be)

% Fit latent class regression model for capture-recapture data with covariates
%
% *** INPUT ***
% Y      : vector of capture configurations
% J      : number of lists
% C      : number of latent classes
% X      : 3-way matrix of covariates
% Bm,G   : matrices used to formulate the model for the conditional distribution
% Pi     : starting values of joint distribution of latent class and response
%
% *** OUTPUT ***
% lk,bic : maximum log-likelihood and bic criterion
% N      : estimate of N
% be     : estimate of the parameters for the regression model
% Pi     : final estiamte of joint distribution of latent class and response
% P      : final estimate of manifest probabilities

% Preliminaries
  [Co,Ma,G] = matbin(Bm); N = sum(sum(Y));
  k = 2^J-1;
  Co = blkdiag([-eye(C-1) eye(C-1)],kron(eye(C),Co));
  np = size(X,2); nc = size(Co,1);
  Mat = eye(C); Ma1 = Mat; Ma1(C,:) = []; Ma2 = Mat; Ma2(1,:) = []; 
  Ma = [kron([Ma1; Ma2],ones(1,2^J)); kron(eye(C),Ma)];
  Gt = eye(C); Gt(:,1) = [];
  Gt = Gt-ones(C,C-1)/C;
  G = [kron(Gt,ones(2^J,1)) kron(eye(C),G)];
  H = inv(G'*G)*G';
  L = kron(eye(C),ones(1,2^J)); M = kron(ones(1,C),eye(2^J));
  At = eye(k+1);
  A = kron(ones(1,C),At); a = sum(A);
% startint values
  ta = sum(Y,1);
%  Y(:,find(ta<eps)) = 10^-6;
  s = size(Y,1); n = sum(sum(Y));
  if nargin < 6,
    la = zeros(J,1);
    for j = 1:J,
      laj = margin(ta,2*ones(1,J),j)/n; la(j) = laj(2)/sum(laj);
    end
    la = log(la./(1-la));
    lm = la-2; lM = la+2;
    S = sq(J); La = zeros(s,C*2^J); 
    for c = 1:C,
      ind = (c-1)*2^J+1:c*2^J;
      la = lm+(lM-lm)*c/(C+1);
      la = exp(la)./(1+exp(la));
      La(:,ind) = ones(s,1)*prod(S*diag(la)+(1-S)*diag(1-la),2)';
    end
    Pi = La/C;
    Eta = log(Pi*Ma')*Co';
    be = zeros(np,1); nt = 0;
    for i = 1:s,
      Xi = X(:,:,i); etai = Eta(i,:)'; 
      yi = Y(i,:); ni = sum(yi);
      bei = pinv(Xi'*Xi)*Xi'*etai;
      be = be+bei*ni/n;
    end
  end
  for i = 1:s,
    Xi = X(:,:,i); etai = Xi*be;
    Pi(i,:) = binve(Co,Ma,G,etai)';
  end
  Ga = log(Pi)*H'; Eta = log(Pi*Ma')*Co';
% compute log-likelihood
  P = Pi*A'; q = sum(P,2);
  lk = sum(sum(Y.*log(P)));
  disp([0 lk]);
% iterate
  lko = lk-1; it = 0; de = 1;
  while (abs(lk-lko)/abs(lko)>10^-8 & it<10000) | it<1,
    it = it+1;
% *** E-step ***
    M = zeros(s,C*2^J);
    for i = 1:s,
      yi = Y(i,:);
      mi = (yi./P(i,:))'; 
      mi = diagv(kron(ones(C,1),mi),Pi(i,:)');
      M(i,:) = mi';
    end
    t = sum(sum(M));
% *** M-step ***
% compute complete log-likelihood
    lkc = sum(sum(M.*log(Pi)));
% iterate
    lkco = lkc-1; itc = 0;
    while abs(lkc-lkco)/abs(lkco)>10^-9 & itc<25,
      itc = itc+1;
      sc = zeros(np,1); Fc = zeros(np);
      for i = 1:s,
        Xi = X(:,:,i); pivi = Pi(i,:)';
        Omi = diag(pivi)-pivi*pivi';
        Ri = inv(Co*diagv(1./(Ma*pivi),Ma*diagv(pivi,G)));
        mi = M(i,:)'; ti = sum(mi);
        sc = sc+Xi'*Ri'*G'*(mi-ti*pivi);
        Fc = Fc+ti*Xi'*Ri'*G'*Omi*G*Ri*Xi/t;
      end
      coe = max((3-itc)*(it==1),1);
      be = be+pinv(Fc)*sc/t/coe;
      for i = 1:s,
        Xi = X(:,:,i); etai = Xi*be;
        Pi(i,:) = binve(Co,Ma,G,etai)';
      end
      Ga = log(Pi)*H'; Eta = log(Pi*Ma')*Co';
      lkco = lkc; lkc = sum(sum(M.*log(Pi)));
    end
% Compute log-likelihood
    P = Pi*A';
    lko = lk; lk = sum(sum(Y.*log(P)));
% output
    disp([it lk lk-lko abs((lk-lko)/lko) (lk-lko)/de]);
    de = lk-lko;
  end