function [lk,bic,N,be,Pi,P,wref,Pcref,F] = emcontinc(Y,J,C,X,Bm,be,Xref)

% Fit latent class regression model for capture-recapture data with covariates
%
% *** INPUT ***
% Y         : vector of capture configurations
% J         : number of lists
% C         : number of latent classes
% X         : 3-way matrix of covariates
% Bm        : matrices used to formulate the model for the conditional distribution
% be (opt)  : starting values of the vector of regression parameters
% Xref (opt): design matrix for a reference subject
%
% *** OUTPUT ***
% lk   : maximum log-likelihood and bic criterion
% bic  : Bayesian information criterion
% N    : estimate of N
% be   : estimate of the parameters for the regression model
% se   : standard errors for be
% Pi   : final estiamte of joint distribution of latent class and response
% P    : final estimate of manifest probabilities
% wref : estimates of class weights for a reference subject (all covariate equal to 0)
% Pcref: estimates of conditional probabilities for the reference subject

% Preliminaries
  mod = 'mar';   % use 'log' for log-linear effect or 'mar' for marginal ones
  [Coc,Mac,Gc] = matbin(Bm,mod);
  k = 2^J-1;
  Co = blkdiag([-eye(C-1) eye(C-1)],kron(eye(C),Coc));
  np = size(X,2); nc = size(Co,1); ncc = size(Coc,1);
  Mat = eye(C); Ma1 = Mat; Ma1(C,:) = []; Ma2 = Mat; Ma2(1,:) = []; 
  if mod == 'log',
    Ma = [kron([Ma1; Ma2],[1 zeros(1,2^J-1)]); kron(eye(C),Mac)];
  elseif mod == 'mar',
    Ma = [kron([Ma1; Ma2],ones(1,2^J)); kron(eye(C),Mac)];
  end
  Gt = eye(C); Gt(:,1) = [];
  Gt = Gt-ones(C,C-1)/C;
  G = [kron(Gt,ones(2^J,1)) kron(eye(C),Gc)];
  H = inv(G'*G)*G';
  L = kron(eye(C),ones(1,2^J)); M = kron(ones(1,C),eye(2^J));
  At = eye(k+1); At(1,:) = [];
  A = kron(ones(1,C),At); a = sum(A);
% starting values
  ta = sum(Y,1);
  s = size(Y,1); n = sum(sum(Y));
  if nargin < 6,
    be = [];
  end
  if isempty(be),
    la = zeros(J,1);
    for j = 1:J,
      laj = margin(ta,2*ones(1,J),j)/n; la(j) = laj(2)/sum(laj);
    end
    la = log(la./(1-la));
    lm = la-2; lM = la+2;
    S = sq(J); La = zeros(s,C*2^J); 
    for c = 1:C,
      ind = (c-1)*2^J+1:c*2^J;
      la = lm+(lM-lm)*c/(C+1);
      la = exp(la)./(1+exp(la));
      La(:,ind) = ones(s,1)*prod(S*diag(la)+(1-S)*diag(1-la),2)';
    end
    Pi = La/C;
    Eta = log(Pi*Ma')*Co';
    be = zeros(np,1); nt = 0;
    for i = 1:s,
      Xi = X(:,:,i); etai = Eta(i,:)'; 
      yi = Y(i,:); ni = sum(yi);
      bei = pinv(Xi'*Xi)*Xi'*etai;
      be = be+bei*ni/n;
    end
  end
  if nargin<7,
    Xref = X(:,:,1); Xref(:,C+C*J:end) = 0;
  end
  etaref = Xref*be;
  wref = exp([0; cumsum(etaref(1:C-1))]); wref = wref/sum(wref);
  Pcref = zeros(C,J);
  for c = 1:C,
    indp = (c-1)*2^J+1:c*2^J;
    inde = (C-1)+(ncc*(c-1)+1:ncc*c);
    [pivc,err] = binve(Coc,Mac,Gc,etaref(inde));
    if err, disp('inversion error for a reference subject'); end
    for j = 1:J,
      ve = margin(pivc,2*ones(1,J),j);
      Pcref(c,j) = ve(2);
    end
  end
  for i = 1:s,
    Xi = X(:,:,i); etai = Xi*be;
    if mod == 'mar',
      wei = exp([0; cumsum(etai(1:C-1))]); wei = wei/sum(wei);
      pivi = zeros(C*2^J,1);
      for c = 1:C,
        indp = (c-1)*2^J+1:c*2^J;
        inde = (C-1)+(ncc*(c-1)+1:ncc*c);
        [pivc,err] = binve(Coc,Mac,Gc,etai(inde));
        if err, disp('inversion error at the beginning'); end
        Pi(i,indp) = wei(c)*pivc';
      end
    elseif mod == 'log',
      Pi(i,:) = binve(Co,Ma,G,etai)';
    end
  end
  Ga = log(Pi)*H'; Eta = log(Pi*Ma')*Co';
  P = Pi*A'; q = sum(P,2);
  disp(wref);
  disp(Pcref);
% compute log-likelihood
  P = Pi*A'; q = sum(P,2);
  lk = sum(sum(Y(:,2:2^J).*log(diagv(1./q,P))));
  disp([0 lk]);
% iterate
  lko = lk-1; N = n; it = 0; de = 1;
  while (abs(lk-lko)/abs(lko)>10^-8 & it<5000) | it<1,
    it = it+1;
% *** E-step ***
    M = zeros(s,C*2^J);
    for i = 1:s,
      yi = Y(i,2:2^J);
      mi = [sum(yi)/q(i); (yi./P(i,:))']; 
      mi = diagv(kron(ones(C,1),mi),Pi(i,:)');
      M(i,:) = mi';
    end
    t = sum(sum(M));
% *** M-step ***
% compute complete log-likelihood
    lkc = sum(sum(M.*log(Pi)));
% iterate
    lkco = lkc-1; itc = 0;
    while abs(lkc-lkco)/abs(lkco)>10^-8 & itc<25,
      itc = itc+1;
      sc = zeros(np,1); Fc = zeros(np);
      tv = sum(M,2); DMG = (M-diagv(tv,Pi))*G; DPiM = 1./(Pi*Ma');
      for i = 1:s,
        Xi = X(:,:,i); pivi = Pi(i,:)';
        Omi = diag(pivi)-pivi*pivi';
        Ri = inv(Co*diagv(DPiM(i,:)',Ma*diagv(pivi,G)));
        sc = sc+Xi'*Ri'*DMG(i,:)';
        Fc = Fc+tv(i)*Xi'*Ri'*G'*Omi*G*Ri*Xi/t;
      end
      half = 1; ith = 0;
      beo = be; lkco = lkc; 
      dbe = pinv(Fc)*sc/t; 
      if max(abs(dbe))>0.5,
        dbe = dbe/max(abs(dbe))*0.5;
      end
      while half,
        ith = ith+1; half = 0;
        if ith==20,
          be = beo;
        else
          be = beo+dbe/ith;
        end
        for i = 1:s,
          etai = X(:,:,i)*be;
          if mod == 'mar',
            wei = exp([0; cumsum(etai(1:C-1))]); wei = wei/sum(wei);
            for c = 1:C,
              indp = (c-1)*2^J+1:c*2^J;
              inde = (C-1)+(ncc*(c-1)+1:ncc*c);
              [pivc,err] = binve(Coc,Mac,Gc,etai(inde));
              Pi(i,indp) = wei(c)*pivc';
            end
          elseif mod == 'log',
            Pi(i,:) = binve(Co,Ma,G,etai)';
          end
        end
        Ga = log(Pi)*H'; Eta = log(Pi*Ma')*Co';
        lkc = sum(sum(M.*log(Pi)));
        if (lkc-lkco<-10^-8*abs(lkco) & ith<2) | (any(any(Pi<10^-250)) & ith<20),
          half = 1;
          disp(min(min(Pi)));
        end
      end
%      disp([itc lkc lkc-lkco min(min(Pi))]);
    end
% Compute log-likelihood
    P = Pi*A'; q = sum(P,2);
    lko = lk; lk = sum(sum(Y(:,2:2^J).*log(diagv(1./q,P))));
    No = N; N = sum(sum(Y,2)./q);
% Display output
%    disp('-------------------------------------------------------------------------------------------');
    disp([it lk lk-lko N N-No abs((lk-lko)/lko) (lk-lko)/de]);
%    disp('-------------------------------------------------------------------------------------------');
    de = lk-lko;
  end
% Bayesian information criterion
  bic = -2*lk+log(n)*(np);
% Information matrix and standard errors
  Hti = [-ones(k-1,1) eye(k-1)]; Gti = Hti'*inv(Hti*Hti');
  F = zeros(np);
  It = eye(k+1); It(1,:) = [];
  for i = 1:s,
    Xi = X(:,:,i); yi = Y(i,:); ni = sum(yi);
    pivi = Pi(i,:)'; pi = P(i,:)'; qi = q(i);
    Di = (1/qi^2)*kron(ones(C),It'*(inv(diag(pi/qi))-ones(k))*It);
    Ri = inv(Co*diagv(1./(Ma*pivi),Ma*diagv(pivi,G)));
    F = F+ni*Xi'*Ri'*G'*diag(pivi)*Di*diag(pivi)*G*Ri*Xi/n;
  end
  F = n*F;
  if rcond(n*F)<10^-15,
    disp('singular information matrix');
  end
% Parameter estimates for a reference subject
  etaref = Xref*be;
  wref = exp([0; cumsum(etaref(1:C-1))]); wref = wref/sum(wref);
  Pcref = zeros(C,J);
  for c = 1:C,
    indp = (c-1)*2^J+1:c*2^J;
    inde = (C-1)+(ncc*(c-1)+1:ncc*c);
    [pivc,err] = binve(Coc,Mac,Gc,etaref(inde));
    if err, disp('inversion error for a reference subject'); end
    for j = 1:J,
      ve = margin(pivc,2*ones(1,J),j);
      Pcref(c,j) = ve(2);
    end
  end